@@ -36,23 +36,24 @@ from .options import (
     command_options_option,
     force_option,
     group_filters_option,
+    main_path_option,
     manifest_option,
-    path_option,
     paths_argument,
     process_command,
     process_command_options,
-    process_path,
+    process_main_path,
     process_paths,
     projects_option,
     reverse_option,
     update_option,
+    ws_path_option,
 )
 from .remote import remote
 
 _LOGGING_FORMAT = "%(name)s %(levelname)s %(message)s"
 
 
-@click.group(context_settings=dict(help_option_names=["-h", "--help"]))
+@click.group(context_settings={"help_option_names": ["-h", "--help"]})
 @click.option("-v", "--verbose", count=True)
 @click.version_option(package_name="git-ws")
 @click.pass_context
@@ -71,23 +72,44 @@ def main(ctx=None, verbose=0):
 
 
 @main.command()
-@path_option()
+@main_path_option()
+@ws_path_option()
 @manifest_option(initial=True)
 @group_filters_option(initial=True)
 @update_option()
 @force_option()
 @pass_context
-def init(context, path=None, manifest_path=None, group_filters=None, update: bool = False, force: bool = False):
+def init(
+    context,
+    ws_path=None,
+    main_path=None,
+    manifest_path=None,
+    group_filters=None,
+    update: bool = False,
+    force: bool = False,
+):
     """
-    Initialize Git Workspace at PATH or current directory.
+    Initialize Git Workspace *with* or *without* main project.
 
-    The actual directory MUST be a valid git clone, which has been
-    either created by 'git init' or 'git clone'.
+    *with* main project: run `git ws init` in the directory of an actual git clone OR
+    run `git ws init PATH_TO_MAIN_GIT_CLONE`. The git clone becomes the main project
+    in both cases.
+
+    *without* main project: run `git ws init` in the intended workspace directory.
+    No main git project is needed.
+
+    In any of these cases a manifest file (i.e. `git-ws.toml`) is required.
+    It can be created via `git ws manifest create`.
     """
     with exceptionhandling(context):
-        path = process_path(path)
+        main_path = process_main_path(main_path)
         gws = GitWS.init(
-            main_path=path, manifest_path=manifest_path, group_filters=group_filters, force=force, secho=context.secho
+            path=ws_path,
+            main_path=main_path,
+            manifest_path=manifest_path,
+            group_filters=group_filters,
+            force=force,
+            secho=context.secho,
         )
         click.secho(f"Workspace initialized at {str(resolve_relative(gws.path))!r}.")
         if update:
@@ -112,7 +134,8 @@ def deinit(context, prune: bool = False, force: bool = False):
 
 @main.command()
 @click.argument("url")
-@path_option()
+@main_path_option()
+@ws_path_option(nomain=True)
 @manifest_option(initial=True)
 @group_filters_option(initial=True)
 @click.option("--revision", help="Revision to be checked out. Tag, Branch or SHA")
@@ -122,7 +145,8 @@ def deinit(context, prune: bool = False, force: bool = False):
 def clone(
     context,
     url,
-    path=None,
+    ws_path=None,
+    main_path=None,
     manifest_path=None,
     group_filters=None,
     revision=None,
@@ -132,13 +156,14 @@ def clone(
     """
     Create a git clone from URL and initialize Git Workspace.
 
-    PATH is optional. If not specified `REPONAME/REPONAME/` by default.
+    MAIN_PATH is optional. If not specified `REPONAME/REPONAME/` by default.
     """
     with exceptionhandling(context):
-        path = process_path(path)
+        main_path = process_main_path(main_path)
         gws = GitWS.clone(
             url,
-            main_path=path,
+            path=ws_path,
+            main_path=main_path,
             manifest_path=manifest_path,
             group_filters=group_filters,
             revision=revision,
