@@ -17,6 +17,7 @@
 """Collection Of All Exceptions Which Might Occur."""
 
 from pathlib import Path
+from typing import Optional
 
 
 class UninitializedError(RuntimeError):
@@ -29,10 +30,13 @@ class UninitializedError(RuntimeError):
 class InitializedError(RuntimeError):
     """Git Workspace Has Already Been Initialized."""
 
-    def __init__(self, path, main_path):
-        super().__init__(
-            f"git workspace has already been initialized at {str(path)!r} with main repo at {str(main_path)!r}."
-        )
+    def __init__(self, path: Path, main_path: Optional[Path]):
+        msg = f"git workspace has already been initialized at {str(path)!r}"
+        if main_path:
+            msg = f"{msg} with main repo at {str(main_path)!r}."
+        else:
+            msg = f"{msg}."
+        super().__init__(msg)
         self.path = path
         self.main_path = main_path
 
@@ -60,19 +64,20 @@ class ManifestExistError(RuntimeError):
 
 
 class OutsideWorkspaceError(RuntimeError):
-    """Project Is Located Outside Of Workspace."""
+    """Reference to Outside Of Workspace."""
 
-    def __init__(self, path, project_path):
-        super().__init__(f"Project {str(project_path)!r} is located outside {str(path)!r}.")
+    def __init__(self, workspace_path, path, what):
+        super().__init__(f"{what} {str(path)!r} is refers outside of workspace ({str(workspace_path)!r}).")
+        self.workspace_path = workspace_path
         self.path = path
-        self.project_path = project_path
 
 
 class WorkspaceNotEmptyError(RuntimeError):
     """Workspace Is Not Empty."""
 
-    def __init__(self, path):
-        super().__init__(f"Workspace {str(path)!r} is not an empty directory.")
+    def __init__(self, path, items):
+        items = ", ".join(str(item) for item in items)
+        super().__init__(f"Workspace {str(path)!r} is not an empty directory. It contains: {items}.")
         self.path = path
 
 
@@ -151,3 +156,20 @@ class GitTagExistsError(RuntimeError):
     def __init__(self, tag):
         super().__init__(f"tag {tag} already exists")
         self.tag = tag
+
+
+class NoMainError(RuntimeError):
+    """Workspace Has No Main Project."""
+
+    def __init__(self):
+        super().__init__("Workspace has been initialized from manifest only, without a main project.")
+
+
+class NoAbsUrlError(RuntimeError):
+    """No Relative URL Possible."""
+
+    def __init__(self, project_name: str):
+        super().__init__(
+            "Absolute URL required. Please specify an absolute 'url' or a 'sub_url'"
+            f" with a 'remote' for {project_name!r}."
+        )
